# Compatible with python 2.* version
import sys
import json
import time
import threading
import random
from iotconnect import IoTConnectSDK
from datetime import datetime

#Get your "ENV" and "CPID" from the portal key vaults module or visit https://help.iotconnect.io SDK section.

env = "<< Environment >>" 
uniqueId = "<< Device UniqueID >>"
cpId = "<< CPID >>"

interval = 1

tProcess = None
dataArray = []
d2cMsg = None

def callbackMessage(msg):
    global d2cMsg
    print("\n--- Command Message Received ---")
    cmdType = None
    if msg != None and len(msg.items()) != 0:
        cmdType = msg["cmdType"] if msg["cmdType"] != None else None
    
    # Other Command
    if cmdType == "0x01":
        print(json.dumps(msg))
    
    # Firmware Upgrade
    if cmdType == "0x02":
        data = msg["data"] if msg["data"] != None else None
        if data != None:
            print(json.dumps(data))
            d2cMsg = {
                "guid" : data['guid'],
                "st" : 3,
                "msg" : ""
            }

def callbackTwinMessage(msg):
    if msg:
        print("\n--- Twin Message Received ---")
        print(json.dumps(msg))

def sendBackToSDK(sdk, dataArray):
    global tProcess
    sdk.SendData(dataArray)
    time.sleep(interval)
    tProcess = None

def processData():
    try:
        # Sample Data
        return {
            "Temperature": random.randint(20,50),
            "Humidity": random.randint(10,50),
            "Gyroscope": {
                'x': random.randint(1,10),
                'y': random.randint(1,10),
                'z': random.randint(1,10),
            }
        }
        #return None
    except:
        return None

def main():
    global tProcess, dataArray, d2cMsg
    try:
        with IoTConnectSDK(cpId, uniqueId, callbackMessage, callbackTwinMessage, env) as sdk:
            try:
                #sdk.GetAttributes()
                #sdk.UpdateTwin({key}, {value})
                # Note : cmdType = "0x02" - Firmware Upgrade
                # sdk.SendACK(11, {}) # Send acknowledgement
                # 11 : Firmware acknowledgement
                # e.g : acknowledgement data
                # {
                #    "guid":"78978304-9e8a-4789-ab28-a81955a82a18",
                #    "st": 3, //sent = 0,pending = 1,skipped = 2,success = 3,failed = 4
                #    "msg": "" //Your custom message
                # }
                while True:
                    data = processData()
                    if data != None:
                        dObj = {
                            "uniqueId": uniqueId,
                            "time": datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.000Z"),
                            "data": data
                        }
                        dataArray.append(dObj)
                    
                    if tProcess == None and len(dataArray) > 0:
                        tProcess = threading.Thread(target = sendBackToSDK, args = [sdk, dataArray])
                        tProcess.setName("SEND")
                        tProcess.daemon = True
                        tProcess.start()
                        tProcess.join(1)
                    
                    if d2cMsg != None:
                        sdk.SendACK(11, d2cMsg) # 11 : Firmware acknowledgement
                        d2cMsg = None
            except KeyboardInterrupt:
                sys.exit(0)
    except Exception as ex:
        print(ex.message)
        sys.exit(0)

if __name__ == "__main__":
    main()
